from .base import *  # NOQA

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql',
        'HOST': '127.0.0.1',
        'NAME': 'postgres',
        'USER': 'postgres',
        'PASSWORD': 'mayanuserpass',
    }
}
